import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.fail;

import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

class FrequencyAnalysisTest {

	@BeforeAll
	static void setUpBeforeClass() throws Exception {
	}

	@AfterEach
	void tearDown() throws Exception {
	}

	@Test
	void testTwoCharSequencesAAA() {
		String input = "aaa";
		List<String> output = FrequencyAnalysis.twoCharSequences(input);
		assertEquals("aa", output.get(0));
		assertEquals("aa", output.get(1));
		assertEquals(2, output.size());
	}

	@Test
	void testTwoCharSequencesAABA() {
		String input = "aaba";
		List<String> output = FrequencyAnalysis.twoCharSequences(input);
		assertEquals("aa", output.get(0));
		assertEquals("ab", output.get(1));
		assertEquals("ba", output.get(2));
		assertEquals(3, output.size());
	}

	@Test
	void testIncrementFromListStartingEmpty() {
		List<String> inputs = new ArrayList<>();
		inputs.add("aa");
		inputs.add("cd");
		inputs.add("aa");
		inputs.add("ba");

		Map<String, Double> counts = new HashMap<>();
		FrequencyAnalysis.incrementFromList(inputs, counts);

		assertEquals(2.0, counts.get("aa"), 1e-6);
		assertEquals(1.0, counts.get("cd"), 1e-6);
		assertEquals(1.0, counts.get("ba"), 1e-6);
	}

	@Test
	void testIncrementFromListComplex() {
		List<String> inputs = new ArrayList<>();
		inputs.add("aa");
		inputs.add("cd");
		inputs.add("aa");
		inputs.add("ba");

		Map<String, Double> counts = new HashMap<>();
		FrequencyAnalysis.incrementFromList(inputs, counts);
		inputs.add("jd");
		FrequencyAnalysis.incrementFromList(inputs, counts);

		assertEquals(4.0, counts.get("aa"), 1e-6);
		assertEquals(2.0, counts.get("cd"), 1e-6);
		assertEquals(2.0, counts.get("ba"), 1e-6);
		assertEquals(1.0, counts.get("jd"), 1e-6);
	}

	@Test
	void testNormalize() {
		Map<String, Double> counts = new HashMap<>();
		counts.put("aa", 4.0);
		counts.put("cd", 2.0);
		counts.put("ba", 2.0);
		counts.put("jd", 1.0);
		FrequencyAnalysis.normalize(counts);

		double totalCount = 9.0;

		assertEquals(4.0 / totalCount, counts.get("aa"), 1e-6);
		assertEquals(2.0 / totalCount, counts.get("cd"), 1e-6);
		assertEquals(2.0 / totalCount, counts.get("ba"), 1e-6);
		assertEquals(1.0 / totalCount, counts.get("jd"), 1e-6);
	}

	@Test
	void testCalculateFrequencies() {
		Map<String, Double> freqs = null;
		try {
			freqs = FrequencyAnalysis.calculateFrequencies("dictionary.txt");
		} catch (FileNotFoundException e) {
			fail();
		}

		assertEquals(0.75, freqs.get("aa"), 1e-6);
		assertEquals(0.25, freqs.get("ab"), 1e-6);
		assertEquals(2, freqs.size()); // should just have 2 keys

	}

}
